from typing import List

from serial_json import DataClass, field

from ..schedule import Schedule


__all__ = ['DataClass', 'Message', 'Error', 'Quit', 'Update', 'RunCommand', 'ScheduleCommand',
           'RunningSchedule', 'ListSchedules', 'StopSchedule']


class Message(DataClass):
    message: str


class Error(DataClass):
    message: str


class Quit(DataClass):
    pass


class Update(DataClass):
    pass


class RunCommand(DataClass):
    callback_name: str
    args: tuple = field(default_factory=tuple)
    kwargs: dict = field(default_factory=dict)


class ScheduleCommand(DataClass):
    name: str
    schedule: Schedule
    callback_name: str
    args: tuple = field(default_factory=tuple)
    kwargs: dict = field(default_factory=dict)


class RunningSchedule(DataClass):
    name: str
    schedule: Schedule


class ListSchedules(DataClass):
    schedules: List[RunningSchedule] = field(default_factory=list)


class StopSchedule(DataClass):
    name: str




from .utils import ScheduleError
from .schedule import Schedule

try:
    from .server import get_server, set_server, Scheduler, Client, \
        Message, Error, Quit, Update, RunCommand, ScheduleCommand, RunningSchedule, ListSchedules, StopSchedule
except (ImportError, Exception) as err:
    error = err
    class ClassEnvironmentError:
        def __new__(cls, *args, **kwargs):
            raise EnvironmentError('Dependencies not installed! '
                                   'Libraries pydantic and pydantic_decoder are required!') from error

    get_server = ClassEnvironmentError
    set_server = ClassEnvironmentError
    Scheduler = ClassEnvironmentError
    Client = ClassEnvironmentError

    Message = ClassEnvironmentError
    Error = ClassEnvironmentError
    Quit = ClassEnvironmentError
    Update = ClassEnvironmentError
    RunCommand = ClassEnvironmentError
    ScheduleCommand = ClassEnvironmentError
    RunningSchedule = ClassEnvironmentError
    ListSchedules = ClassEnvironmentError
    StopSchedule = ClassEnvironmentError



from .messages import Message, Error, Quit, Update, RunCommand, ScheduleCommand, \
    RunningSchedule, ListSchedules, StopSchedule
from .srv import get_server, set_server, Scheduler
from .client import Client
